//genesis

float	CA_FRACT	=	1.000
float   PI      =   3.14159
float   CM = 0.01           // Farads/m^2
float   RA = 0.5            // Ohms m
float   RM = 0.2            // Ohms m^2
float   EREST_ACT = -0.070  // Volts
int i

	float OUT_CA = 1.5 // mM
	float IN_CA = 0.001 // mM, 1 uM is an intermediate value.
						// We'll try others.
	float OUT_NA = 145 // mM
	float IN_NA = 12 // mM
	float OUT_K = 4 // mM
	float IN_K = 155 // mM
	float PR_Ca= 10.6
	float PR_Na= 1.0
	float PR_K= 1.0

/**********************************************************************
** Sets up NMDA- & non-NMDA-synaptic firing experiment for Ca 
**	concentrations in a dendritic spine
**  by E. De Schutter, November 1990
**	Minor modifications by U.S.Bhalla June 1995, for Genesis 2
**  compatibility
** Uses granule.p file or spine.p model file (for spine.p file: 
**	decrease all (NON_)NMDA_G values by factor 10, very high RN!).
**
** Reference: Holmes WR & Levy WB: Insights into associative
**	long-term potentiation from computational models of NMDA
**	receptor-mediated caclcium influx and intracellular calcium
**	concentrations changes.  J. Neurophysiol. 63, 1148-1168, 1990.  
**********************************************************************/

/*********************************************************************/

float FbyRT = 1.0/0.02673
function nv(charge,v)
	float charge
	float v
	return {exp {-charge * FbyRT * v}}
end

function find_pratio(v)
	float v
	float temp = {nv 1.0 {v}}
	if (temp == 1.0)
		return 0.0
	end
	float Nar = PR_Na * (IN_NA - OUT_NA * temp) / (1.0 - temp)
	float Kr = PR_K * (IN_K - OUT_K * temp) / (1.0 - temp)
	float Car 
	temp = {nv 2.0 {v}}
	Car = 4.0 * PR_Ca * (IN_CA - OUT_CA * temp) / (1.0 - temp)

	return {-v * Car / (Car + Nar + Kr)}
end

function make_the_spine_prototypes
/* separate function so we can have local variables */
/* This script uses the *compt feature of the cell_reader.  All types of
**  compartments, with their channels and diffusion elements attached to them
**  in a subtree (including the messages), are created in a library.  The
**  cell_reader then copies to over 1000 compartments in the final model
**  and scales all fields correctly for surface and volume (except for
**  gbar, gmax, Len or Vol fields which have a negative value). */
/* we need 3 types of comps: one with [buffer]=100uM (base), one with 
** [buffer]=200uM (head2) and idem + NMDA & non-NMDA receptor (head1) */

	echo Making spine library.

	float len,dia,surf,vol,kPump
	int i,ndivs
	//cable parameters
		CM = 0.01
		RM = 1.20
		RA = 0.70
		EREST_ACT = -0.070
	//minimal concentration
	// Changed by Upi to 0.08 uM
//		float   STEADY_CA   =   0.000020
		float   STEADY_CA   =   0.000080
	//diffusion constant
		float   DIFFC_CA    =   0.6e-9
	//effective fraction of compartment volume available for diffusion
		float   DIFFVOLFR   =   1.0
	//for Holmes Levy spine model tau is determined by 1/kp with kp in m/s
		float   PUMP_KP     =   0.7e-6
	//total buffer concentration
		float   BUFFER_HTOT  =   0.200
		float   BUFFER_BTOT  =   0.100
	//forward rate constant for buffer
		float   BUFFER_FRATE =   500000.0
	//backward rate constant for buffer
		float   BUFFER_BRATE =   500.0
	/* About the non-NMDA and NMDA-channels.  These are here
	**	implemented by the manuelconduct object.  Though Holmes and
	**	Levy refer to this scheme, they do not implement it exactly.
	**	The non-NMDA-channel follows a slightly different scheme, which
	**	I "massaged" into the manuelconduct form, the NMDA-channel is
	**	correct.  For both channels the total number of receptors
	**	should be known, as activation is expressed relative to that
	**	total number.  As there is no evidence for receptor-binding
	**	saturation (the binding is supposed to be infinitely fast), I
	**	take the total number of receptors to be 20000000, so that each
	**	transmitter pulse (200 units) binds a fraction of 0.00001. 
	**  The large number of receptors reduces the stiffness a bit.
	** These channels may be activated by setting Z = Z + 0.00001.
	**  In practice however a larger pulse has to be given to
	**	compensate for the small differences in channel kinetics.  */
	//alpha1 NMDA receptor/channel kinetics, changed to get it
	//	into the manuelconduct form.  This is OK as long as
	//	alpha1+alpha2=340 1/s (p 1153).
		float	NMDA_ALPHA1	=	170.0
	//alpha2 NMDA receptor/channel kinetics
		float	NMDA_ALPHA2	=	170.0
	//beta NMDA receptor/channel kinetics
		float	NMDA_BETA	=	0.600
	//k-1 NMDA receptor/channel kinetics
		float	NMDA_K	=	20.0
	// Upi addition:
		float	NMDA_TAU_D = 2e-6
	//single NMDA channel conductance (negative value: no scaling)
		float	NMDA_G	=	-50.0e-12
	//reversal potential NMDA channel
		float	NMDA_EK	=	0.0
	//NMDA Mg block A-factor (scaled to mM units)
		float	MGBLOCK_A	=	8.80
	//NMDA Mg block B-factor
		float	MGBLOCK_B	=	0.0125	//scaled for V instead of mV
	//Mg concentration
		float	STEADY_MG	=	1.2
	//reversal potential non-NMDA channel
		float	NON_NMDA_EK	=	0.0
	//alpha1 non-NMDA receptor/channel kinetics: same trick as for
	//	the NMDA channel, here this creates however a somewhat
	//	different equation from the Holmes and Levy one.
		float	NON_NMDA_ALPHA1	=	1200.0
	//alpha2 non-NMDA receptor/channel kinetics
		float	NON_NMDA_ALPHA2	=	800.0
	//beta non-NMDA receptor/channel kinetics
		float	NON_NMDA_BETA	=	2000.0
	//k-1 non-NMDA receptor/channel kinetics
		float	NON_NMDA_K	=	2000.0
	//single non-NMDA channel conductance (negative value: no scaling)
		float	NON_NMDA_G	= -5.0e-12
	// since we are doing a spine here, reduce NON_NMDA_G by 10x
//		NON_NMDA_G	= NON_NMDA_G/10.0
	
	/* make 200uM [buffer] head prototype */
	len = 0.05e-6
	dia = 0.55e-6
	surf = len * dia * PI
	if (!{exists head2comp})
		create	compartment head2comp
	end
	setfield head2comp \
		Cm		{CM * surf} \		// F
		Ra		{4.0*RA*len / (dia*dia*PI)} \	// ohm
		Em  	{EREST_ACT} \			// V
		Rm		{RM / surf} \ 		// ohm
		dia		{dia} \
		len		{len} \
		inject	0.0 
	addfield head2comp Shape
	setfield head2comp Shape   "cylinder"

	pushe head2comp

    /* create the diffusion element.  Vol is volume of the element, 
	**  Len is the lenght along the axis of diffusion (one-dimensional)  */
	/* note that Len and Vol will be scaled by the cell_reader, (unless
	**  a negative value is specified), which assumes that Len scales 
	**  with the length of the compartment */
	vol = DIFFVOLFR * len * dia * dia * PI / 4.0
	if (!{exists Ca_pool})
		create	difpool	Ca_pool
	end
	setfield Ca_pool \
		Co      {STEADY_CA}  \  // mM
    	Cmin    {STEADY_CA}  \   // mM
    	D       {DIFFC_CA} \    // D in m^2/s
    	Len     {len} \         // m
    	Vol     {vol}          // m^3
	/* messages outside of the prototype definitions (between neighbouring
	**  difpools in this case) need to be send to the cell_reader by the
	**  set -env mechanisms.  An exception are the (R)AXIAL messages which 
	**  the cell_reader knows how to handle. */
	addfield Ca_pool addmsg1
	addfield Ca_pool addmsg2
	setfield Ca_pool \
		addmsg1	"-/Ca_pool	./Ca_pool	DIFFUSE	Len Vol Co" \
		addmsg2	"./Ca_pool	-/Ca_pool	DIFFUSE	Len Vol Co"
	
	if (!{exists Ca_buffer})
		create poolbuffer Ca_buffer
	end
	setfield Ca_buffer \
    	Btot    {BUFFER_HTOT} \     // mM
    	kBf     {BUFFER_FRATE} \    // 1/mM.s
    	kBb     {BUFFER_BRATE}     // 1/s
	addmsg	Ca_pool		Ca_buffer	CONCEN	Co
	addmsg	Ca_buffer	Ca_pool		BUFFER	kBf kBb Btot Bfree
	
	pope 	//back to library level
	
	/* make 100uM [buffer] base prototype for heads */
	if (!{exists headcomp})
		copy head2comp	headcomp
	end
	setfield headcomp/Ca_buffer \
    	Btot    {BUFFER_BTOT}      // mM

	/* make 100uM [buffer] base prototype for dendrites */
	if (!{exists basecomp})
		copy head2comp	basecomp
	end
	setfield basecomp/Ca_buffer \
    	Btot    {BUFFER_BTOT}      // mM
	
	/* make 100uM [buffer] base prototype for tails */
	if (!{exists tailcomp})
		copy basecomp	tailcomp
	end
	
	/* make 200uM [buffer] NMDA+non-NMDA head prototype */
	if (!{exists head1comp})
		copy head2comp	head1comp
	end
	pushe head1comp

	/* Described by manuelconduct object.  Internally field H
	**	corresponds to AR* and field Z to AR */
	if (!{exists NMDA})
		create manuelconduct	NMDA
	end
	setfield NMDA \
		tau_g	{NMDA_ALPHA1/NMDA_BETA} \
		tau_h	{1/NMDA_ALPHA2} \
		tau_d	{NMDA_TAU_D} \  /* infinitely fast removal: 2 usec */
		K_r		{NMDA_K/400000} \ /* immediate rec binding */
		gmax	{20000000 * NMDA_G} \ /* normalized by # of receptors */
		Ek		{NMDA_EK}  		//V

	if (!{exists Mg_NMDA_block})
		create	Mg_block	Mg_NMDA_block
	end
	setfield Mg_NMDA_block \
		KMg_A	{MGBLOCK_A} \
		KMg_B	{MGBLOCK_B} \
		CMg		{STEADY_MG} \
		Ek		{NMDA_EK} \ 		//V
		Zk		2
	/* block of NMDA-current by Mg: */
	addmsg	NMDA	Mg_NMDA_block	CHANNEL1	Gk	
	/* Ca inflow through NMDA-channel to Ca-pool; */
	// First, set up the permeability ratio table.
	int N_PR_ENTRIES = 2000
	float MIN_V = -0.1	// -100 mV
	float MAX_V = 0.1
	float D_V = (MAX_V - MIN_V)/N_PR_ENTRIES
	// If we were to do this right, we would have a 2-D table based
	// also on the Ca concentrations. Erik's model says that this
	// is not a big factor, so lets see if we can duplicate these
	// results

	if (!{exists pratio}) 
		create table pratio
	end
	call pratio TABCREATE {N_PR_ENTRIES} {MIN_V} {MAX_V}

	float v
	for (i = 0 ; i <= N_PR_ENTRIES; i = i + 1)
		v = MIN_V + i * D_V
		setfield pratio table->table[{i}] {find_pratio {v}}
	end

	// Hook all the pieces together


	addmsg pratio/.. pratio INPUT Vm // Get the Ca curr
	addmsg Mg_NMDA_block pratio PRD Gk	// Scale by Gk to get curr
	addmsg pratio Ca_pool INJECT 1 output // feed the curr to the Ca_pool

	if (!{exists non_NMDA})
		create manuelconduct	non_NMDA
	end
	setfield non_NMDA \
		tau_g	{NON_NMDA_ALPHA1/NON_NMDA_BETA} \
		tau_h	{1/NON_NMDA_ALPHA2} \
		tau_d	{0.000001} \  /* infinitely fast removal */
		K_r		{NON_NMDA_K/1000000} \  /* immediate receptor binding */
		gmax	{20000000 * NON_NMDA_G}\/* normalized by # receptors */
		Ek	{NON_NMDA_EK} 		//V

	pope 	//back to library level

	/* As kPump is constant, we do not need a separate pump element.
	**	For convenience we use Ca_buffer as the src of the PUMP msg.
	** The Holmes and Levy pump rate is dependent on the surface to volume
	**	ratio of the compartment, for a cylindrical comp this is equal to
	**	4/dia */
	kPump = 4 * PUMP_KP / 0.55e-6
	addmsg	head1comp/Ca_buffer	head1comp/Ca_pool	PUMP	{kPump} {STEADY_CA}
	addmsg	head2comp/Ca_buffer	head2comp/Ca_pool	PUMP	{kPump} {STEADY_CA}
	addmsg	headcomp/Ca_buffer	headcomp/Ca_pool	PUMP	{kPump} {STEADY_CA}
	kPump = 4 * PUMP_KP / 0.10e-6
	addmsg	basecomp/Ca_buffer	tailcomp/Ca_pool	PUMP	{kPump} {STEADY_CA}
	kPump = 4 * PUMP_KP / 1.00e-6
	addmsg	basecomp/Ca_buffer	basecomp/Ca_pool	PUMP	{kPump} {STEADY_CA}

	/* connect channels to compartment */
	/* The VOLTAGE messages to (non_)NMDA are needed only when you
	**  want to output Ik, here they are not used to speed things up */
//	addmsg	head1comp	head1comp/non_NMDA	VOLTAGE	Vm  
	addmsg	head1comp/non_NMDA	head1comp	CHANNEL Gk	Ek
//	addmsg	head1comp	head1comp/NMDA	VOLTAGE	Vm
//	addmsg	head1comp	head1comp/Ca_fract_tab	INPUT	Vm
	addmsg	head1comp	head1comp/Mg_NMDA_block	VOLTAGE	Vm
	addmsg	head1comp/Mg_NMDA_block	head1comp	CHANNEL Gk	Ek

	copy /library/head1comp/non_NMDA /library
	setfield /library/non_NMDA gmax 1e-7
	addfield /library/non_NMDA addmsg1
	setfield /library/non_NMDA addmsg1 ". .. CHANNEL Gk Ek"
end
